@@ -22,7 +22,8 @@ from zlib import compress
 import os
 import sys
 
-EXCEPTIONAL_URL = "http://api.getexceptional.com/api/errors"
+EXCEPTIONAL_URL = "http://api.exceptional.io/api/errors"
+
 
 class Exceptional(object):
     """Extension for tracking application errors with Exceptional.
@@ -62,13 +63,15 @@ class Exceptional(object):
 
         if "EXCEPTIONAL_API_KEY" in app.config:
             app.config.setdefault("EXCEPTIONAL_COOKIE_FILTER", None)
-            app.config.setdefault("EXCEPTIONAL_ENVIRONMENT_FILTER", ["SECRET_KEY"])
+            app.config.setdefault("EXCEPTIONAL_ENVIRONMENT_FILTER",
+                    ["SECRET_KEY"])
             app.config.setdefault("EXCEPTIONAL_HEADER_FILTER", None)
             app.config.setdefault("EXCEPTIONAL_PARAMETER_FILTER", None)
             app.config.setdefault("EXCEPTIONAL_SESSION_FILTER", None)
-            app.config.setdefault("EXCEPTIONAL_HTTP_CODES", set(xrange(400, 418)))
+            app.config.setdefault("EXCEPTIONAL_HTTP_CODES",
+                    set(xrange(400, 418)))
             app.config.setdefault("EXCEPTIONAL_DEBUG_URL", None)
-            self.__protocol_version = 5 # Using zlib compression.
+            self.__protocol_version = 5  # Using zlib compression.
 
             if app.debug:
                 self.url = app.config["EXCEPTIONAL_DEBUG_URL"]
@@ -93,6 +96,27 @@ class Exceptional(object):
         else:
             app.logger.warning("Missing 'EXCEPTIONAL_API_KEY' configuration.")
 
+    @staticmethod
+    def context(data=None, **kwargs):
+        """Add extra context data to the current tracked exception. The context
+        data is only valid for the current request. Multiple calls to this
+        method will update any existing context with new data.
+
+        :param data: Default ``None``. A dictionary of context data.
+        :param kwargs: A series of keyword arguments to use as context data.
+        """
+        context = getattr(_request_ctx_stack.top, "exceptional_context", None)
+
+        if context is None:
+            context = {}
+            setattr(_request_ctx_stack.top, "exceptional_context", context)
+
+        if data is not None:
+            context.update(data)
+
+        if len(kwargs):
+            context.update(kwargs)
+
     @staticmethod
     def publish(config, traceback):
         """Publish the given traceback directly to Exceptional. This method is
@@ -127,6 +151,7 @@ class Exceptional(object):
                        Accepts either :class:`flask.Config` or the object
                        types allowed by :meth:`flask.Config.from_object`.
         """
+        context = getattr(_request_ctx_stack.top, "exceptional_context", None)
         app = Flask(__name__)
         exceptional = Exceptional()
 
@@ -143,6 +168,7 @@ class Exceptional(object):
 
         @app.route("/exception")
         def exception():
+            setattr(_request_ctx_stack.top, "exceptional_context", context)
             message = "Congratulations! Your application is configured for Exceptional error tracking."
 
             raise Exception(message)
@@ -190,7 +216,8 @@ class Exceptional(object):
     def _post_data(self, context=None, traceback=None):
         """POST data to the the Exceptional API. If DEBUG is True then data is
         sent to ``EXCEPTIONAL_DEBUG_URL`` if it has been defined. If TESTING is
-        true, error data is stored in the global ``flask.g.exceptional`` variable.
+        true, error data is stored in the global ``flask.g.exceptional``
+        variable.
 
         :param context: Default ``None``. The current application context.
         :param traceback: Default ``None``. The exception stack trace.
@@ -204,17 +231,22 @@ class Exceptional(object):
         }
 
         if context:
-            request_data = self.__get_request_data(app, context.request, context.session)
+            request_data = self.__get_request_data(app, context.request,
+                    context.session)
+            context_data = getattr(context, "exceptional_context", None)
         else:
             request_data = None
+            context_data = None
 
         traceback = traceback or tbtools.get_current_traceback()
         exception_data = self.__get_exception_data(traceback)
         encode_basestring = json.encoder.encode_basestring
 
         def _encode_basestring(value):
-            if isinstance(value, str) and json.encoder.HAS_UTF8.search(value) is not None:
-                value = value.decode("utf-8", "replace") # ensure the decode succeeds.
+            if isinstance(value, str) and \
+                    json.encoder.HAS_UTF8.search(value) is not None:
+                value = value.decode("utf-8",
+                        "replace")  # ensure the decode succeeds.
 
             replace = lambda match: json.encoder.ESCAPE_DCT[match.group(0)]
 
@@ -226,7 +258,8 @@ class Exceptional(object):
                 "application_environment": application_data,
                 "client": client_data,
                 "request": request_data,
-                "exception": exception_data
+                "exception": exception_data,
+                "context": context_data
             }, ensure_ascii=False).encode("utf-8")
         finally:
             json.encoder.encode_basestring = encode_basestring
@@ -252,7 +285,7 @@ class Exceptional(object):
                         raise
             except URLError:
                 message = "Unable to connect to %s. See \
-http://status.getexceptional.com for details. Error data:\n%s" % (self.url, error_data)
+http://status.exceptional.io for details. Error data:\n%s" % (self.url, error_data)
                 self.app.logger.warning(message, exc_info=True)
 
     @staticmethod
@@ -292,7 +325,8 @@ http://status.getexceptional.com for details. Error data:\n%s" % (self.url, erro
 
         return {
             "framework": "flask",
-            "env": Exceptional.__filter(app, environment, "EXCEPTIONAL_ENVIRONMENT_FILTER"),
+            "env": Exceptional.__filter(app, environment,
+                "EXCEPTIONAL_ENVIRONMENT_FILTER"),
             "language": "python",
             "language_version": sys.version.replace('\n', ''),
             "application_root_directory": app.root_path
@@ -346,8 +380,9 @@ http://status.getexceptional.com for details. Error data:\n%s" % (self.url, erro
                 parameters[key] = [file.filename for file in value]
 
         if request.cookies:
-            cookies = Exceptional.__filter(app, request.cookies, "EXCEPTIONAL_COOKIE_FILTER")
-            headers = Headers(request.headers) # Get a mutable dictionary.
+            cookies = Exceptional.__filter(app, request.cookies,
+                    "EXCEPTIONAL_COOKIE_FILTER")
+            headers = Headers(request.headers)  # Get a mutable dictionary.
             cookie = SimpleCookie()
 
             for key, value in cookies.iteritems():
@@ -358,12 +393,17 @@ http://status.getexceptional.com for details. Error data:\n%s" % (self.url, erro
             headers = request.headers
 
         return {
-            "session": Exceptional.__filter(app, session, "EXCEPTIONAL_SESSION_FILTER"),
+            "session": Exceptional.__filter(app, session,
+                "EXCEPTIONAL_SESSION_FILTER"),
             "remote_ip": request.remote_addr,
-            "parameters": Exceptional.__filter(app, parameters, "EXCEPTIONAL_PARAMETER_FILTER"),
-            "action": request.endpoint.split('.', 1)[-1] if request.endpoint else None,
+            "parameters": Exceptional.__filter(app, parameters,
+                "EXCEPTIONAL_PARAMETER_FILTER"),
+            "action": request.endpoint.split('.', 1)[-1] if request.endpoint \
+                    else None,
             "url": request.url,
             "request_method": request.method,
-            "controller": request.blueprint if hasattr(request, "blueprint") else request.module,
-            "headers": Exceptional.__filter(app, headers, "EXCEPTIONAL_HEADER_FILTER")
+            "controller": request.blueprint if hasattr(request, "blueprint") \
+                    else request.module,
+            "headers": Exceptional.__filter(app, headers,
+                "EXCEPTIONAL_HEADER_FILTER")
         }
